#include <config.h>

#include "datpacker.h"
#include "th10.h"

#include <stdlib.h>
#include <unistd.h>

char library_error[LIBRARY_ERROR_SIZE] = { 0 };

static void
print_usage()
{
	puts("Usage: datpacker [OPTION]... ARCHIVE FILE...");
	puts("OPTION can be:");
	puts("  -h           prints the usage");
	puts("  -v VERSION   defaults to 11");
}

int
main(int argc, char* argv[])
{
	FILE* archive;
	void* private;
	thdat_fptrs* fptrs = NULL;
	unsigned int version = 11;
	unsigned int i;
	int opt;

	opterr = 0;
	while ((opt = getopt(argc, argv, "hv:")) != -1) {
		if (opt == 'h') {
			print_usage();
			return EXIT_SUCCESS;
		} else if (opt == 'v') {
			version = strtol(optarg, NULL, 10);
		} else {
			print_usage();
			return EXIT_FAILURE;
		}
	}

	if ((optind + 2) > argc) {
		print_usage();
		return EXIT_FAILURE;
	}

	switch (version) {
	case 10:
	case 11:
		fptrs = &th10dat_fptrs;
		break;
	}

	if (!fptrs) {
		fprintf(stderr, "datpacker: unsupported version\n");
		return EXIT_FAILURE;
	}

	archive = fopen(argv[optind], "wb");
	if (!archive) {
		fprintf(stderr, "datpacker: couldn't open %s for writing\n", argv[optind]);
		return EXIT_FAILURE;
	}

	private = fptrs->open(archive);
	if (!private) {
		fprintf(stderr, "datpacker: %s\n", library_error);
		fclose(archive);
		return EXIT_FAILURE;
	}

	for (i = optind + 1; i < argc; ++i) {
		FILE* fd = fopen(argv[i], "rb");
		if (!fd) {
			fprintf(stderr, "datpacker: couldn't open %s for reading\n", argv[i]);
			fclose(archive);
			return EXIT_FAILURE;
		}
		printf("%s... ", argv[i]);
		fflush(stdout);
		if (fptrs->write(private, fd, argv[i]) == -1) {
			fprintf(stderr, "datpacker: %s\n", library_error);
			fclose(fd);
			fclose(archive);
			return EXIT_FAILURE;
		}
		printf("OK!\n");
		fclose(fd);
	}

	if (fptrs->close(private) == -1) {
		fprintf(stderr, "datpacker: %s\n", library_error);
		fclose(archive);
		return EXIT_FAILURE;
	}

	fclose(archive);

	return EXIT_SUCCESS;
}
