#include <config.h>
#include <string.h>

unsigned int
th_get_cryprm_index(const char *name)
{
	char index = 0;
	while(*name) index += *name++;
	return index & 7;
}

struct {
	unsigned char key;
	unsigned char step;
	unsigned int block;
	unsigned int limit;
} g_cryprm[] = {
	/* key  step  block  limit */
	{ 0x1b, 0x37, 0x40,  0x2800 },
	{ 0x51, 0xe9, 0x40,  0x3000 },
	{ 0xc1, 0x51, 0x80,  0x3200 },
	{ 0x03, 0x19, 0x400, 0x7800 },
	{ 0xab, 0xcd, 0x200, 0x2800 },
	{ 0x12, 0x34, 0x80,  0x3200 },
	{ 0x35, 0x97, 0x80,  0x2800 },
	{ 0x99, 0x37, 0x400, 0x2000 }
};

void
th_encrypt(const unsigned char* in, unsigned char* out, int size, unsigned char key, const unsigned char step, int block, int limit)
{
	const unsigned char* iend = NULL;
	const unsigned char* ilimit = NULL;
	const unsigned char* ip = NULL;
	const unsigned char* irealend = in + size;
	unsigned int rest;

	rest = size % block;
	if (rest >= block / 4)
		rest = 0;
	rest += size % 2;
	size -= rest;

	for (iend = in + size, ilimit = in + limit; in < iend && in < ilimit; in += block) {
		int increment;
		if (iend - in < block)
			block = iend - in;
		increment = block / 2;
		if (block % 2)
			increment++;

		for (ip = in + block - 1; ip > in; ip -= 2) {
			*out = *ip ^ key;
			*(out + increment) = *(ip - 1) ^ ((key + step * increment));
			out++;
			key += step;
		}

		if (block % 2) {
			*out = *ip ^ key;
			key += step;
		}

		/* Increment key by step * (block / 2). */
		key += step * increment;
		/* Increment out by (block / 2). */
		out += increment;
	}

	if (in != irealend)
		memcpy(out, in, irealend - in);
}
